# coding: utf-8

"""
    Curia Platform API

    These are the docs for the curia platform API. To test, generate an authorization token first.  # noqa: E501

    OpenAPI spec version: 3.16.0
    
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""

import pprint
import re  # noqa: F401

import six

class BooleanExpression(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """
    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        'enabled': 'bool',
        'logic': 'str',
        'expressions': 'list[BooleanExpression]',
        'field': 'str',
        'operator': 'object',
        'value': 'object'
    }

    attribute_map = {
        'enabled': 'enabled',
        'logic': 'logic',
        'expressions': 'expressions',
        'field': 'field',
        'operator': 'operator',
        'value': 'value'
    }

    def __init__(self, enabled=None, logic=None, expressions=None, field=None, operator=None, value=None):  # noqa: E501
        """BooleanExpression - a model defined in Swagger"""  # noqa: E501
        self._enabled = None
        self._logic = None
        self._expressions = None
        self._field = None
        self._operator = None
        self._value = None
        self.discriminator = None
        if enabled is not None:
            self.enabled = enabled
        if logic is not None:
            self.logic = logic
        if expressions is not None:
            self.expressions = expressions
        if field is not None:
            self.field = field
        if operator is not None:
            self.operator = operator
        if value is not None:
            self.value = value

    @property
    def enabled(self):
        """Gets the enabled of this BooleanExpression.  # noqa: E501


        :return: The enabled of this BooleanExpression.  # noqa: E501
        :rtype: bool
        """
        return self._enabled

    @enabled.setter
    def enabled(self, enabled):
        """Sets the enabled of this BooleanExpression.


        :param enabled: The enabled of this BooleanExpression.  # noqa: E501
        :type: bool
        """

        self._enabled = enabled

    @property
    def logic(self):
        """Gets the logic of this BooleanExpression.  # noqa: E501


        :return: The logic of this BooleanExpression.  # noqa: E501
        :rtype: str
        """
        return self._logic

    @logic.setter
    def logic(self, logic):
        """Sets the logic of this BooleanExpression.


        :param logic: The logic of this BooleanExpression.  # noqa: E501
        :type: str
        """
        allowed_values = ["and", "or", "not"]  # noqa: E501
        if logic not in allowed_values:
            raise ValueError(
                "Invalid value for `logic` ({0}), must be one of {1}"  # noqa: E501
                .format(logic, allowed_values)
            )

        self._logic = logic

    @property
    def expressions(self):
        """Gets the expressions of this BooleanExpression.  # noqa: E501


        :return: The expressions of this BooleanExpression.  # noqa: E501
        :rtype: list[BooleanExpression]
        """
        return self._expressions

    @expressions.setter
    def expressions(self, expressions):
        """Sets the expressions of this BooleanExpression.


        :param expressions: The expressions of this BooleanExpression.  # noqa: E501
        :type: list[BooleanExpression]
        """

        self._expressions = expressions

    @property
    def field(self):
        """Gets the field of this BooleanExpression.  # noqa: E501


        :return: The field of this BooleanExpression.  # noqa: E501
        :rtype: str
        """
        return self._field

    @field.setter
    def field(self, field):
        """Sets the field of this BooleanExpression.


        :param field: The field of this BooleanExpression.  # noqa: E501
        :type: str
        """

        self._field = field

    @property
    def operator(self):
        """Gets the operator of this BooleanExpression.  # noqa: E501


        :return: The operator of this BooleanExpression.  # noqa: E501
        :rtype: object
        """
        return self._operator

    @operator.setter
    def operator(self, operator):
        """Sets the operator of this BooleanExpression.


        :param operator: The operator of this BooleanExpression.  # noqa: E501
        :type: object
        """

        self._operator = operator

    @property
    def value(self):
        """Gets the value of this BooleanExpression.  # noqa: E501


        :return: The value of this BooleanExpression.  # noqa: E501
        :rtype: object
        """
        return self._value

    @value.setter
    def value(self, value):
        """Sets the value of this BooleanExpression.


        :param value: The value of this BooleanExpression.  # noqa: E501
        :type: object
        """

        self._value = value

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value
        if issubclass(BooleanExpression, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, BooleanExpression):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other
